// ExamplePlugin.cpp : Defines the exported functions for the MysticThumbs plugin.
//

#include "ExamplePlugin.h"
#include "MysticThumbsPlugin.h"
#include <Shlwapi.h>
#include <math.h>
#include <new>

// Define a name and the extensions supported by this plugin
static const LPCWSTR s_name = L"Example Plugin";
static const LPCWSTR s_extensions[] = { L".mtp" };

// Generate a GUID using Visual Studio Tools menu -> Create GUID and copy and paste here for uniqueness.
// {4501A6B8-65EF-42A4-8AB1-F5F9FB23A91F}
static const GUID s_guid = { 0x4501a6b8, 0x65ef, 0x42a4, { 0x8a, 0xb1, 0xf5, 0xf9, 0xfb, 0x23, 0xa9, 0x1f } };

#ifndef PI
#define PI 3.14159265358979f
#endif

class CExamplePlugin : public IMysticThumbsPlugin
{
public:

    CExamplePlugin()
    {
        // Called when the plugin is initially created.
        // It may not necesserally be used immediately, it may just be an object to check the interface information so be careful about allocating anything here.
    }

private:

    virtual LPWSTR GetName() override
    {
        return StrDupW(s_name);
    }

    virtual LPCGUID GetGuid() override
    {
        return &s_guid;
    }

    virtual unsigned int GetExtensionCount() override
    {
        return ARRAYSIZE(s_extensions);
    }

    virtual LPWSTR GetExtension(IN unsigned int index) override
    {
        return StrDupW(s_extensions[index]);
    }

    virtual void Destroy() override
    {
        // Delete this instance
        this->~CExamplePlugin();
        CoTaskMemFree(this);
    }

    virtual bool PingImage(IN IStream* pStream, OUT MysticThumbsPluginPing& _ping) override
    {
        auto& ping = (MysticThumbsPluginPing2&)_ping;
        // Any ping members untouched will be ignored
        ping.width = ping.isQuickView && ping.requiredWidth ? ping.requiredWidth : 256;
        ping.height = ping.isQuickView && ping.requiredHeight ? ping.requiredHeight : 256;
        ping.bitDepth = 32;
        return true;
    }

    virtual unsigned char* GenerateImage(IN IStream* pStream, IN unsigned int desiredSize, IN unsigned int flags, OUT bool& hasAlpha, OUT unsigned int& width, OUT unsigned int& height) override
    {
        // pStream is pointing to the head of the file stream on entry so you can begin by reading the file header etc.

        // for this example we are just going to generate a test-pattern image of size desiredSize x desiredSize, with transparency
        width = desiredSize;
        height = desiredSize;
        hasAlpha = true;
        const unsigned int numChannels = 4; // We are outputting alpha, so 4 channels

        // Allocate the image memory, Use LocalAlloc with LMEM_FIXED for this.
        // Don't use LMEM_MOVEABLE as it adds extra overhead and semantically requires a LocalLock() which MysticThumbs does not do.
        unsigned char* image = (unsigned char*)LocalAlloc(LMEM_FIXED, (SIZE_T)width * height * numChannels);

        // Decode
        if(unsigned char* ptr = image) {
            for(unsigned int y = 0; y < height; ++y) {
                float fy = ((float)y / (height - 1)) * 2 - 1;
                for(unsigned int x = 0; x < width; ++x) {
                    // For this example, calculate some RGBA values since we don't actually have a real file stream
                    // Instead of calculating values here we would normally read from pStream to gather the image data
                    float fx = ((float)x / (width - 1)) * 2 - 1;
                    float r = (1 + sinf(fx * PI)) * 0.5f;
                    float g = (1 + cosf(fy * PI)) * 0.5f;
                    float b = (1 + sinf(fx * fy * PI)) * 0.5f;
                    float a = sqrtf(fx * fx + fy * fy);

                    // Fill in the channel information for this pixel
                    // Instead of this:
                    *ptr++ = (unsigned char)(r * 255);
                    *ptr++ = (unsigned char)(g * 255);
                    *ptr++ = (unsigned char)(b * 255);
                    *ptr++ = (unsigned char)(a * 255);
                }
            }
        }
        return image;

        // return NULL if there is an error
        // make sure to set hasAlpha, width and height on successful completion
    }
};



// Return the version of the plugin compiled.
extern "C" EXAMPLEPLUGIN_API int Version()
{
    return MYSTICTHUMBS_PLUGIN_VERSION;
}

// Initialize the plugin. Perform any work here that needs to be done for all instances such as loading DLLs or initializing globals.
extern "C" EXAMPLEPLUGIN_API bool Initialize()
{
    // For this example we don't need to do anything
    return true;
}

// Shutdown the plugin. Clean everything up.
extern "C" EXAMPLEPLUGIN_API bool Shutdown()
{
    // For this example we don't need to do anything
    return true;
}

// Create an instance of a thumbnail plugin
extern "C" EXAMPLEPLUGIN_API IMysticThumbsPlugin * CreateInstance()
{
    // Be sure to use CoTaskMemAlloc and placement new to create your object so it is within
    // the processes memory space or it may end up being in protected memory if the DLL shuts down.

    // Note that the IMysticThumbsPlugin::Destroy method takes care of deleting this object when MysticThumbs is finished with it.
	CExamplePlugin* plugin = (CExamplePlugin*)CoTaskMemAlloc(sizeof(CExamplePlugin));
	new(plugin) CExamplePlugin();
    return plugin;
}


EXAMPLEPLUGIN_API BOOL APIENTRY DllMain(HINSTANCE hModule, DWORD ul_reason_for_call, void* lpReserved)
{
    UNREFERENCED_PARAMETER(lpReserved);

    switch(ul_reason_for_call)
    {
        case DLL_PROCESS_ATTACH:
            break;

        case DLL_PROCESS_DETACH:
            break;

        case DLL_THREAD_ATTACH:
            break;

        case DLL_THREAD_DETACH:
            break;
    }
    return TRUE;
}
