/////////////////////////////////////////////////////////////////////////////
// 
// Header for MysticThumbs plugins
// 
// Copyright 2011 MysticCoder
// http://mysticcoder.net/mysticthumbs
//
// This file is the primary interface header for MysticThumbs plugins.
// It should not be modified or redistributed in any way.
// Please see the MysticThumbs license.
// 
// 2024-12-27 - Bumped MYSTICTHUMBS_PLUGIN_VERSION to 2.
// Added Version() DLL function for plugins to report their version.
// Work on flags to support single checkerboard and canvas.
// Updated to MysticThumbsPluginPing2 for Ping().
// 
/////////////////////////////////////////////////////////////////////////////

#pragma once

#ifndef _MysticThumbsPlugin_h_
#define _MysticThumbsPlugin_h_


// Header version
// When implementing version 2 or newer plusins, use this to return from the DLL Version() function.
// Version 2: rearranged MysticThumbsPluginFlags.
#define MYSTICTHUMBS_PLUGIN_VERSION 2


#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

#ifndef INOUT
#define INOUT
#endif

// Include Windows headers that we need
#include <Windows.h>
#include <Objidl.h>



/////////////////////////////////////////////////////////////////////////////
// 
// Ping information, used when MysticThumbs is requesting image information
// 
/////////////////////////////////////////////////////////////////////////////

struct MysticThumbsPluginPing
{
	unsigned int width;
	unsigned int height;
	unsigned int bitDepth;
};

// for MYSTICTHUMBS_PLUGIN_VERSION >= 2 Ping()
struct MysticThumbsPluginPing2 : public MysticThumbsPluginPing
{
	bool isQuickView;					// This is fetching for QuickView
    unsigned int requiredWidth;			// either the thumbnail width or if isQuickView then the window width
    unsigned int requiredHeight;		// either the thumbnail width or if isQuickView then the window height
};


/////////////////////////////////////////////////////////////////////////////
// 
// Flags passed to IMysticThumbsPlugin::GenerateImage()
// 
// These are only hints. Generally you should create a transparent 32 bit
// image since MysticThumbs does the heavy lifting on appearance etc.
// 
/////////////////////////////////////////////////////////////////////////////

enum MysticThumbsPluginFlags : unsigned int
{
	// NOTE: these have changed since version 2. They are still hints but are now in different orders. Recompile may be needed.
	// Transparency flags, one of the following:
	MT_Transparency_Disable         = 0x00000001, // opaque
	MT_Transparency_Canvas			= 0x00000002, // Used to be MT_Transparency_Checkerboard2.
	MT_Transparency_Transparent     = 0x00000004,
	MT_Transparency_Checkerboard    = 0x00000008,
	//MT_Transparency_Mask			= 0x0000000f, // all bits

    // NOTE: these have changed since version 2. They are still hints but are now in different orders. Recompile may be needed.
    // Embedded thumbnail flags, one of the following:
	MT_EmbeddedThumb_Never          = 0x00000010,
	MT_EmbeddedThumb_Always         = 0x00000020,
	MT_EmbeddedThumb_IfLarger       = 0x00000040,

	// Scale to fit is requested. This is handled by MysticThumbs, but the hint is here.
	MT_ScaleUp						= 0x00000100,
};
DEFINE_ENUM_FLAG_OPERATORS(MysticThumbsPluginFlags)

/////////////////////////////////////////////////////////////////////////////
// 
// Derive your plugin class from this interface and implement all methods
// 
/////////////////////////////////////////////////////////////////////////////

struct IMysticThumbsPlugin
{
protected:
	virtual ~IMysticThumbsPlugin() {}

public:
	/// Destroy this plugin instance
	virtual void Destroy() = 0;

	/// Identify this plugin by name
	/// Use LocalAlloc to allocate the string, for example by using StrDupW(), because LocalFree will be called on the returned pointer.
	/// Do NOT return a static string.
	virtual LPWSTR GetName() = 0;

	// Identify plugin by GUID for uniqueness
	virtual LPCGUID GetGuid() = 0;

	// List extensions supported
	virtual unsigned int GetExtensionCount() = 0;

	/// Use LocalAlloc to allocate the returned extension
	virtual LPWSTR GetExtension(IN unsigned int index) = 0;

	/// Ping an image to report it's information such as dimensions and bit depth.
	/// Fill in fields that are relevant. If no ping information can be determined do nothing and return false.
	/// Cast MysticThumbsPluginPing to MysticThumbsPluginPing2 or whatever is relevant for your plugin MYSTICTHUMBS_PLUGIN_VERSION.
	virtual bool PingImage(IN IStream* pStream, INOUT MysticThumbsPluginPing& ping) = 0;

	/// Create a bitmap image from the given file stream
	/// returns an array of size length allocated with LocalAlloc that will be freed by MysticThumbs.
	/// if hasAlpha is false then the size of the bitmap should be width * height * 3 in the format RGB
	/// if hasAlpha is true then the size of the bitmap should be width * height * 4 in the format RGBA
	/// flags are of the MysticThumbsPluginFlags set
	virtual unsigned char* GenerateImage(IN IStream* pStream, IN unsigned int desiredSize, IN unsigned int flags, OUT bool& hasAlpha, OUT unsigned int& width, OUT unsigned int& height) = 0;
};


/////////////////////////////////////////////////////////////////////////////
// 
// DLL functions required to be implemented by each plugin
// 
// Note: These these functions should defined extern "C" so they can be found
//       See ExamplePlugin.cpp
// 
/////////////////////////////////////////////////////////////////////////////


// Identify version of plugin used. Will not exist in a version 1 plugin.
// For MYSTICTHUMBS_PLUGIN_VERSION >= 2 implement this and return MYSTICTHUMBS_PLUGIN_VERSION.
typedef int (*MTP_Version)();

// Initialize the plugin. Any initialization required for all instances can be done here.
typedef bool (*MTP_Initialize)();

// Shutdown the plugin. Clean up everything when the plugin is being unloaded.
typedef bool (*MTP_Shutdown)();

// Create an instance of a thumbnail provider
typedef IMysticThumbsPlugin* (*MTP_CreateInstance)();


#endif // _MysticThumbsPlugin_h_
